static char *CopyrightIdentifier(void) { return "@(#)lsansi9t.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }
#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>

#include <unistd.h>	// for read(2)
#include <fcntl.h>

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>

#include <string.h>

#include "GetOpt.h"
#include "MainCmd.h"

#include "ansi9t.h"

static const char *extract(const char *src,unsigned length)
{
	static char buffer[80];
	strncpy(buffer,src,length);
	buffer[length]=0;
	return buffer;
}

static void
dumpAnsiHdrTitle(void)
{
	cerr << "File Identifier  " << " | ";
	cerr << "FSetId" << " | ";
	cerr << "Sec#" << " | ";
	cerr << "Seq#" << " | ";
	cerr << "Gen#" << " | ";
	cerr << "GV" << " | ";
	cerr << "Create" << " | ";
	cerr << "Expire" << " | ";
	cerr << "A" << " | ";
	cerr << "Blocks" << " | ";
	cerr << "System Code  " << "\n";
}

static void
dumpAnsiHdrList(union AnsiHdr *u)
{
	if (!strncmp(u->c.labelid,"HDR",3)
	 && !strncmp(u->c.labelno,"1",1)) {
		cerr << setw(17) << extract(u->hdr1.fileid,17) << " | ";
		cerr << setw(6)  << extract(u->hdr1.filesetid,6) << " | ";
		cerr << setw(4)  << extract(u->hdr1.filesectionnumber,4) << " | ";
		cerr << setw(4)  << extract(u->hdr1.filesequencenumber,4) << " | ";
		cerr << setw(4)  << extract(u->hdr1.generationnumber,4) << " | ";
		cerr << setw(2)  << extract(u->hdr1.generationversionnumber,2) << " | ";
		cerr << setw(6)  << extract(u->hdr1.creationdate,6) << " | ";
		cerr << setw(6)  << extract(u->hdr1.expirationdate,6) << " | ";
		cerr << setw(1)  << extract(u->hdr1.accessibility,1) << " | ";
		cerr << setw(6)  << extract(u->hdr1.blockcount,6) << " | ";
		cerr << setw(13) << extract(u->hdr1.systemcode,13) << "\n";
	}
}

static void
dumpAnsiHdrBrief(union AnsiHdr *u)
{
	if (!strncmp(u->c.labelid,"HDR",3)
	 && !strncmp(u->c.labelno,"1",1)) {
		cerr << extract(u->c.labelid,3);
		cerr << extract(u->c.labelno,1) << ":";	
		cerr << "\tFile identifier:\t \"" << extract(u->hdr1.fileid,17) << "\"\n";
		cerr << "\tFile set identifier:\t \"" << extract(u->hdr1.filesetid,6) << "\"\n";
		cerr << "\tFile section number:\t \"" << extract(u->hdr1.filesectionnumber,4) << "\"\n";
		cerr << "\tFile sequence number:\t \"" << extract(u->hdr1.filesequencenumber,4) << "\"\n";
		cerr << "\tGeneration number:\t \"" << extract(u->hdr1.generationnumber,4) << "\"\n";
		cerr << "\tGeneration version no:\t \"" << extract(u->hdr1.generationversionnumber,2) << "\"\n";
		cerr << "\tCreation date:\t\t \"" << extract(u->hdr1.creationdate,6) << "\"\n";
		cerr << "\tExpiration date:\t \"" << extract(u->hdr1.expirationdate,6) << "\"\n";
		cerr << "\tAccessibility:\t\t \"" << extract(u->hdr1.accessibility,1) << "\"\n";
		cerr << "\tBlock count:\t\t \"" << extract(u->hdr1.blockcount,6) << "\"\n";
		cerr << "\tSystem code:\t\t \"" << extract(u->hdr1.systemcode,13) << "\"\n";
	}
}

static void
dumpAnsiHdrVerbose(union AnsiHdr *u)
{
	int dumpall=0;
	cerr << extract(u->c.labelid,3);
	cerr << extract(u->c.labelno,1) << ":";	
	if (!strncmp(u->c.labelid,"VOL",3) && !strncmp(u->c.labelno,"1",1)) {
		cerr << "\n";
		cerr << "\tVolume identifier:\t \"" << extract(u->vol1.volumeid,6) << "\"\n";
		cerr << "\tAccesibility:\t\t \"" << extract(u->vol1.accessibility,1) << "\"\n";
		cerr << "\tOwner identifier:\t \"" << extract(u->vol1.ownerid,14) << "\"\n";
		cerr << "\tLabel standard version:\t \"" << extract(u->vol1.labelstandardversion,1) << "\"\n";
	}
	else if (!strncmp(u->c.labelid,"HDR",3)
	      || !strncmp(u->c.labelid,"EOF",3)
	      || !strncmp(u->c.labelid,"EOV",3)) {
		if (!strncmp(u->c.labelno,"1",1)) {
			cerr << "\n";
			cerr << "\tFile identifier:\t \"" << extract(u->hdr1.fileid,17) << "\"\n";
			cerr << "\tFile set identifier:\t \"" << extract(u->hdr1.filesetid,6) << "\"\n";
			cerr << "\tFile section number:\t \"" << extract(u->hdr1.filesectionnumber,4) << "\"\n";
			cerr << "\tFile sequence number:\t \"" << extract(u->hdr1.filesequencenumber,4) << "\"\n";
			cerr << "\tGeneration number:\t \"" << extract(u->hdr1.generationnumber,4) << "\"\n";
			cerr << "\tGeneration version no:\t \"" << extract(u->hdr1.generationversionnumber,2) << "\"\n";
			cerr << "\tCreation date:\t\t \"" << extract(u->hdr1.creationdate,6) << "\"\n";
			cerr << "\tExpiration date:\t \"" << extract(u->hdr1.expirationdate,6) << "\"\n";
			cerr << "\tAccessibility:\t\t \"" << extract(u->hdr1.accessibility,1) << "\"\n";
			cerr << "\tBlock count:\t\t \"" << extract(u->hdr1.blockcount,6) << "\"\n";
			cerr << "\tSystem code:\t\t \"" << extract(u->hdr1.systemcode,13) << "\"\n";
		}
		else if (!strncmp(u->c.labelno,"2",1)) {
			cerr << "\n";
			cerr << "\tRecord format:\t\t \"" << extract(u->hdr2.recordformat,1) << "\"\n";
			cerr << "\tBlock length:\t\t \"" << extract(u->hdr2.blocklength,5) << "\"\n";
			cerr << "\tRecord length:\t\t \"" << extract(u->hdr2.recordlength,5) << "\"\n";
			cerr << "\tBuffer offset length:\t \"" << extract(u->hdr2.bufferoffsetlength,2) << "\"\n";
		}
		else {
			cerr << " unknown label number\n";
			dumpall=1;
		}
	}
	else {
		if (!strncmp(u->c.labelid,"UHL",3)
	         || !strncmp(u->c.labelid,"UTL",3)
	         || !strncmp(u->c.labelid,"UVL",3)) {
			cerr << " unknown contents\n";
		}
		else {
			cerr << " unknown label\n";
		}
		dumpall=1;
	}
	if (dumpall) {
		u->buffer[80]=0;
		cerr << "\"" << u->buffer << "\"\n";
	}
}

int
main(int argc,char **argv)
{
	CCOMMAND(argc,argv);

	char *devicename;
	int verbose=0;
	int brief=0;
	int list=0;

	GetOpt getopt(argc,argv,"blv");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'b':
				brief=1;
				break;
			case 'l':
				list=1;
				break;
			case 'v':
				verbose=1;
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (!verbose && !brief && !list) {
		cerr << "Must specify one of -b|l|v\n";
		err=1;
	}

	if (getopt.optind+1 == argc) {
		devicename=argv[getopt.optind];
	}
	else {
		cerr << "Must specify one devicename\n";
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0] << " -b|l|v devicename\n"
		     << flush;
		return 1;
	}

	int fd;
	int markcount=0;
	char buf[64512];

	if ((fd=open(devicename,O_RDONLY)) == -1) {
		cerr << "Can't open " << devicename << "\n" << flush;
	}

	if (list) dumpAnsiHdrTitle();
	while (markcount < 2) {
		int num=read(fd,buf,64512);
		switch (num) {
			case -1:
				cerr << "Read Error\n" << flush;
				exit(1);
			case 0:
				if (verbose) cerr << "Tape Mark\n" << flush;
				++markcount;
				break;
			case 80:
				if (brief)
					dumpAnsiHdrBrief((union AnsiHdr *)buf);
				if (list)
					dumpAnsiHdrList((union AnsiHdr *)buf);
				if (verbose)
					dumpAnsiHdrVerbose((union AnsiHdr *)buf);
				markcount=0;
				break;
			case 2048:
				if (verbose)
					cerr << "Read " << num << "bytes\n" << flush;
				markcount=0;
				break;
			default:
				if (verbose)
					cerr << "Read " << num
					     << "bytes (not ANSI standard length)\n"
					     << flush;
				markcount=0;
				break;
		}
	}
	if (verbose) cerr << "Two Tape Marks == End Of Media\n" << flush;
	exit(0);
}
