static char *CopyrightIdentifier(void) { return "@(#)s15ex9t.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <stdlib.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include <unistd.h>	// for read(2)
#include <fcntl.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>
#include <sys/stat.h>	// for mkdir()

#include <string.h>
#include <ctype.h>

#include "GetOpt.h"
#include "MainCmd.h"

#include "ansi9t.h"
#include "s15ptdrc.h"

static const char *extract(const char *src,unsigned length)
{
	static char buffer[80];
	strncpy(buffer,src,length);
	buffer[length]=0;
	return buffer;
}

static const char *extractSansSpaces(const char *from,unsigned length)
{
	static char buffer[80];
	char *to = buffer;
	while (*from && length--) {
		if (isspace(*from)) ++from;
		else *to++=*from++;
	}
	*to=0;
	return buffer;
}

static void
dumpPatientDrc(InitialString *is,FixedFields *ff,FileNameEntry *fne)
{
	cerr << "PATIENT.DRC Contents:\n";

	cerr << "\tInitialString:\n";
	cerr << "\t\tlength:\t \"" << extract(is->length,4) << "\"\n";
	cerr << "\t\tPatientName:\t \"" << extract(is->u.split.PatientName,InitialStringPatientNameLength) << "\"\n";
	cerr << "\t\tPatientId:\t \"" << extract(is->u.split.PatientId,InitialStringPatientIdLength) << "\"\n";
	cerr << "\t\tArchiveDate:\t \"" << extract(is->u.split.ArchiveDate,InitialStringArchiveDateLength) << "\"\n";
	cerr << "\t\tArchiveTime:\t \"" << extract(is->u.split.ArchiveTime,InitialStringArchiveTimeLength) << "\"\n";

	cerr << "\tFixedFields:\n";
	cerr << "\t\tlength:\t \"" << extract(ff->length,4) << "\"\n";
	cerr << "\t\tPatientName:\t \"" << extract(ff->PatientName,FixedFieldsPatientNameLength) << "\"\n";
	cerr << "\t\tPatientId:\t \"" << extract(ff->PatientId,FixedFieldsPatientIdLength) << "\"\n";
	cerr << "\t\tDob:\t\t \"" << extract(ff->Dob,FixedFieldsDobLength) << "\"\n";
	cerr << "\t\tScanDate:\t \"" << extract(ff->ScanDate,FixedFieldsScanDateLength) << "\"\n";
	cerr << "\t\tAnnotation1:\t \"" << extract(ff->Annotation1,FixedFieldsAnnotation1Length) << "\"\n";
	cerr << "\t\tAnnotation2:\t \"" << extract(ff->Annotation2,FixedFieldsAnnotation2Length) << "\"\n";

	cerr << "\tFileNameEntry table:\n";
	while (fne->length[0] != 0x5e) {
		cerr << "\t\t \""  << extract(fne->string,FileNameEntryLength-4) << "\"\n";
		++fne;
	}
}

int
main(int argc,char **argv)
{
	char *devicename;
	int list=0;
	int verbose=0;
	int unique=0;

	GetOpt getopt(argc,argv,"lv");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'l':
				list=1;
				break;
			case 'v':
				verbose=1;
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind+1 == argc) {
		devicename=argv[getopt.optind];
	}
	else {
		cerr << "Must specify one devicename\n";
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0] << " -l -v devicename\n"
		     << flush;
		return 1;
	}

	static char buf[64512];
	int fd;

	if ((fd=open(devicename,O_RDONLY)) == -1) {
		cerr << "Can't open " << devicename << "\n" << flush;
		exit(1);
	}

	ofstream outstream;
	int filecount=0;
	char *filename=0;
	char *pathname=0;
	int markcount=0;
	int lng=0;
	while (markcount < 2) {
		int num=read(fd,buf,64512);
		switch (num) {
			case -1:
				cerr << "Read Error\n" << flush;
				exit(1);
			case 0:
				if (lng) {
					if (verbose)
						cerr << "Wrote " << lng
						     << "bytes\n" 
						     << flush;
					outstream.close();
					delete[] filename;
					filename=0;
					++filecount;
					lng=0;
				}
				if (verbose) cerr << "Tape Mark\n" << flush;
				++markcount;
				break;
			case 80:
				markcount=0;
				if (!lng) {	// First record, test for label
					union AnsiHdr *u = (union AnsiHdr *)buf;
					if (!strncmp(u->c.labelid,"VOL",3)
					 || !strncmp(u->c.labelid,"EOF",3)
					 || !strncmp(u->c.labelid,"EOV",3)
					 || !strncmp(u->c.labelid,"UTL",3)
					 || !strncmp(u->c.labelid,"UVL",3)) {
						if (verbose) {
							cerr << extract(u->c.labelid,3);
							cerr << extract(u->c.labelno,1) << ":\n" << flush;
						}	
						if (filename) delete[] filename;
						filename=0;
						break;
					}
					else if (!strncmp(u->c.labelid,"UHL",3)) {
						if (verbose) {
							cerr << extract(u->c.labelid,3);
							cerr << extract(u->c.labelno,1) << ":\n" << flush;
						}	
						// ignore
						break;
					}
					else if (!strncmp(u->c.labelid,"HDR",3)) {
						if (verbose) {
							cerr << extract(u->c.labelid,3);
							cerr << extract(u->c.labelno,1) << ":\n" << flush;
						}	
						if (!strncmp(u->c.labelno,"1",1)) {
							ostrstream ost;
							ost << extractSansSpaces(u->hdr1.fileid,17) << ends;
							filename=ost.str();
						}
						// else ignore
						break;
					}
					// unrecognized label, fall through ...
				}
				// assume middle of file, fall through ...
			default:
				if (verbose)
					cerr << "Read " << num
					     << "bytes\n" << flush;
				markcount=0;
				if (!lng) {
					if (filename) {
						if (!strcmp(filename,"PATIENT.DRC")) {
							char *p=buf;
							struct InitialString *is = (InitialString *)p;
							unsigned count=atoi(is->length);
							if (count != InitialStringLength) {
								cerr << "Internal error: fascinating !"
								     << " - initial string length is " << count
								     << " not expected " << InitialStringLength
								     << " - please report to author\n"
								     << flush;
							}
							p+=count;
							struct FixedFields *ff = (FixedFields *)p;
							count=atoi(ff->length);
							if (count != FixedFieldsLength) {
								cerr << "Internal error: fascinating !"
								     << " - fixed fields length is " << count
								     << " not expected " << FixedFieldsLength
								     << " - please report to author\n"
								     << flush;
							}
							p+=count;
							struct FileNameEntry *fne = (FileNameEntry *)p;
							if (verbose) dumpPatientDrc(is,ff,fne);
							// Let's use the PatientId to disambiguate same filenames ...
							ostrstream ost;
							ost << extractSansSpaces(ff->PatientId,FixedFieldsPatientIdLength) << ends;
							if (pathname) delete[] pathname;
							pathname=ost.str();
							if (verbose)
								cerr << "Mkdir " << pathname
								     << "\n" << flush;
							if (mkdir(pathname,0777) == -1 && errno != EEXIST) {
								cerr << "Mkdir failed of " 
								     << pathname << "\n" 
								     << flush;
								exit(1);
							}
						}
						if (pathname) {
							// Make filename more interesting ...
							ostrstream ost;
							ost << pathname << "/" << filename << ends;
							if (filename) delete[] filename;
							filename=ost.str();
						}
						outstream.open(filename);
						if (!outstream.good()) {
							cerr << "Open failed of " 
							     << filename << "\n" 
							     << flush;
							exit(1);
						}
						if (list || verbose)
							cerr << filename << "\n" << flush;
					}
					else {
						cerr << "No ANSI HDR1 label before data file\n" << flush;
						exit(1);
					}
				}
				outstream.write(buf,num);
				lng+=num;
				break;
		}
	}
	if (verbose) cerr << "Two Tape Marks == End Of Media\n" << flush;
	exit(0);
}
