%{
#include <string.h>

int my_inputstr(char *buffer,int max_size)
{
	char lbuf[5];
	int length;
	extern FILE *yyin;

	if (feof(yyin) || fread(lbuf,1,4,yyin) == 0) return 0;
	length=atoi(lbuf)-4;	/* vax string length includes itself */
	if (length > max_size) {
		fprintf(stderr,
			"my_inputstr: Internal error, buffer too small need %u given %u\n",
			length,max_size);
	}
	if (length) length=fread(buffer,1,length,yyin);
	if (length) {
		buffer[length++]='\n';	/* the vax strings don't have this */
		buffer[length]=0;
	}
	return length;
}

#define MY_INPUTBUFFERSIZE 8192
static char my_inputbuffer[8192];
static char *my_inputptr=0;

int my_getc(void) {
	extern FILE *yyin;
	if (!my_inputptr || !*my_inputptr) {
		int length= feof(yyin) ? 0 : my_inputstr(my_inputbuffer,MY_INPUTBUFFERSIZE);
		my_inputptr=length ? my_inputbuffer : 0;
	}
	return my_inputptr ? *my_inputptr++ : EOF;
}

#ifdef YY_INPUT
/* replace flex YY_INPUT macro ... */
#undef YY_INPUT
#define YY_INPUT(buffer,result,max_size) (result=my_inputstr(buffer,max_size))
#else  /* YY_INPUT */

#ifdef yygetc
/* replace MKS lex yygetc macro ... (untested) */
#undef yygetc
#define yygetc(f)	my_getc()
#else /* yygetc */

/* replace getc for AT&T lex input() macro ... */
#ifdef getc
#undef getc
#endif /* getc */
#define getc(f) my_getc()

#endif /* yygetc */

#endif /* YY_INPUT */

#include "y.tab.h"

extern int g_lineno;
%}

%p 3000

whitespace		[ \t]+
comment			!.*
newline			\n

string			\"[^\"]*[\"]
id			[a-zA-Z][a-zA-Z0-9_]*
real			-?[0-9]+\.[0-9]+[eE][-+]?[0-9]+
integer			-?[0-9]+

colon			[:][-]*
comma			[,]
openparen		[(]
closeparen		[)]

%%

{whitespace}		;
{comment}		;
{newline}		{ g_lineno++; }

{string}		{
				yylval.string=strdup(yytext+1);
				if (yylval.string[yyleng-2] == '"')
					yylval.string[yyleng-2]=0;
				return STRING;
			}
{integer}		{
				yylval.string=strdup(yytext);
				return INTEGER;
			}
{real}			{
				yylval.string=strdup(yytext);
				return REAL;
			}

{colon}			return COLON;
{comma}			return COMMA;
{openparen}		return OPENPAREN;
{closeparen}		return CLOSEPAREN;

nucleus			return NUCLEUS;
scan_mode		return SCANMODE;
imaging_sequence	return SEQUENCE;
slice_thickness		return THICKNESS;
off_center_distance	return OFFCENTER;
field_of_view		return FOV;
scan_resolution		return RESOLUTION;
slices_3D		return SLICES3D;
slices_MS		return SLICESMS;
slice_factor		return SLICEFACTOR;
measurements		return MEASUREMENTS;
echos			return ECHOS;
first_echo_time		return TE1;
second_echo_time	return TE2;
SE_repetition_time	return TRSE;
IR_repetition_time	return TRIR;
IR_delay		return TI;
m_orientation		return ORIENTM;
p_orientation		return ORIENTP;
s_orientation		return ORIENTS;
spectral_presaturation	return PRESATSPECTRAL;
enable_presat		return PRESAT;
flow_comp		return FLOWCOMP;
send_coil		return COILTX;
receive_coil		return COILRX;
patient_weight		return WEIGHT;
patient_birth_year	return BIRTHYEAR;
synthesizer_frequency	return FREQSYNTH;
angleRF_partial_flip	return FLIPANGLE;
actual_BW_per_pixel	return BWPERPIXEL;

{id}			return ID;

.			;

%%
