static char *CopyrightIdentifier(void) { return "@(#)s15toraw.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "GetOpt.h"
#include "MainCmd.h"

#include "s15exnmr.h"

int
main(int argc,char **argv)
{
	char *nmrname;
	char *recname;
	char *prefix;
	char *suffix;
	int verbose=0;
	GetOpt getopt(argc,argv,"v");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'v':
				verbose=1;
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind+4 == argc) {
		nmrname=argv[getopt.optind];
		recname=argv[getopt.optind+1];
		prefix =argv[getopt.optind+2];
		suffix =argv[getopt.optind+3];
	}
	else {
		cerr << "Must specify nnnn.NMR,nnnn.REC,prefix & suffix\n";
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0]
		     << " -v nnnn.NMR nnnn.REC prefix suffix\n"
		     << flush;
		return 1;
	}

	if (verbose) cerr << "Parsing NMR file\n" << flush;

	extern FILE *yyin;
	if ((yyin=fopen(nmrname,"r")) == 0) {
		cerr << "Can't open " << nmrname << " for read\n";
		return 1;
	}

	ExtractedNMRParameters *e = getExtractedNMRParameters();

	fclose(yyin);

	// NB. assume all required parameter strings present ... :(

	// this assumes
	// - the larger resolution is first
	// - it matches display resolution ... :(

	unsigned resolution=atoi(e->resolution[0]);
	unsigned nslices=atoi(e->slicesms);	// hmmm ... :(
	unsigned nechoes=atoi(e->echos);
	unsigned firstslice=nslices/2+1;
	unsigned nentries=nslices*nechoes;

	if (verbose) {
		cerr << "Reading REC file\n" << flush;
		cerr << "\tresolution=\t" << resolution << "\n" << flush;
		cerr << "\tnslices=\t" << nslices << "\n" << flush;
		cerr << "\tnechoes=\t" << nechoes << "\n" << flush;
		cerr << "\tfirstslice=\t" << firstslice << "\n" << flush;
		cerr << "\tnentries=\t" << nentries << "\n" << flush;
	}

	ifstream recstr(recname);

	// The 1st entry is actually the middle slice ...

	unsigned entry;
	for (entry=0; entry < nentries;++entry) {
		unsigned slice=entry/nechoes+firstslice;
		if (slice > nslices) slice-=nslices;
		unsigned echo=entry%nechoes+1;
		ostrstream ostr;
		if (prefix && *prefix) ostr << prefix << ".";
		ostr << setw(1) << echo
		     << "." << setfill('0') << setw(2) << slice;
		if (suffix && *suffix) ostr << "." << suffix;
		ostr << ends;
		char *outname=ostr.str();
		if (verbose) cerr << "Writing " << outname << "\n" << flush;
		ofstream outstr(outname);

		// .REC file is little endian, so is output ...

		unsigned row;
		for (row=0; row < resolution; ++row) {
			static char buffer[2048];
			recstr.read(buffer,resolution*2);
			if (!recstr.good()) {
				cerr << "Read from " << recname
				     << " failed\n" << flush;
				return 1;
			}
			outstr.write(buffer,resolution*2);
			if (!outstr.good()) {
				cerr << "Write to " << outname
				     << " failed\n" << flush;
				return 1;
			}
		}
		outstr.close();
	}
	recstr.close();

	return 0;
}

