.TH DC3CREATE 1 "03 February 1995" "DICOM PS3" "DICOM PS3 - Create dicom file"
.SH NAME
dc3create \- ACR/NEMA DICOM PS3 ... Create dicom file
.SH SYNOPSIS
.HP 10
.B dc3create
[
.B \-b
]
[
.B \-e
]
<
.I inputfilename
>
.I outputfilename
.SH DESCRIPTION
.LP
.B dc3create
creates a dicom 3 or earlier acr/nema file from a description of attributes. The description of attributes matches the output of dc3dump, so that pieces of dumped files can be edited and files reconstructed. Note that the library of dicom attribute handling, dictionary, and transfer syntax routines are NOT used in this program, which allows it to be used as a standalone program to test the library and other utilities.
.LP
Everything needed to create a file is specified in a description read from standard input, apart from some very simple rules that recognize certain value representations for special treatment of the output format (eg. OW, OS and SQ in explicit value representation transfer syntaxes are handled differently).
.LP
The description consists of lines of the following format:
.LP
.B attributes  ...
\n 
\n (group,element) .* VR=<vr> VL=<vl> [vals]
\n (group,element) .* VR=<vr> VL=<vl> <vals>
\n 
Note that the group, element, value length, and values are always in hexadecimal format, whether a leading "0x" is present or not. Hence "14" and "014" are both interpreted as "0x14", or 20 decimal, in these positions (unlike data descriptions). Multiple values should be delimited with a '\\' for string values (CS, DS. etc.) and a ',' for binary attributes (US, etc.).
.LP
The value length is measured against the value length specified, and a warning is issued if they are different. The actual value, rather than the specified value is used, for all except value representations of OW, OS and SQ. This allows the special value of 0xffffffff to be passed (which means "unspecified" to dicom), as well as allowing the actual values to be included on separate lines following the attribute description. Remember that the value length is in bytes, not the number of values, and must be correct if not 0xffffffff, and even the latter is only supported in DICOM for certain encapsulated transfer syntaxes such as JPEG, and not the normal and default transfer syntaxes.
.LP
If the values for an OB or OW attribute are to be specified on suceeding lines, remember to include an empty value list "[]" or the parser will freak.
.LP
.B transfer syntax commands ...
\n 
\n %implicit
\n %explicit
\n %little
\n %big
\n 
Note that the use of big always forces explicit value representation, as there is no valid "implicit value representation big endian" transfer syntax in dicom. These commands are present here, as well as in the command-line options, to allow a change in transfer syntax, such as after a metaheader.
.LP
.B data value format commands ...
\n 
\n %byte
\n %short
\n %word
\n %long
\n 
Specify the interpretation of subsequent data values, as 8, 16 or 32 bit values. Short and word are synonymous. Use of these commands is usually unnecessary, as the presence of an OB or OW value representation in an attribute description automagically sets the data type to byte or short as appropriate. Prior to such an attribute, and in the absence of a data value format command, the default type is byte.
.LP
.B data ...
\n 
\n hexvalue(s)
\n octalvalue(s)
\n decvalue(s)
\n
Note that the C convention of hexvalues starting with "0x", octal values starting with "0", and others being decimal values is followed for data values (unlike attribute descriptions). The data values are included in the output at the current position.
.LP
.SH OPTIONS
Only redirection from standard input or to standard output are
supported. Input and output file names may not
be specified. Errors and warnings goes to standard error.
.LP
The default transfer syntax is implicit value representation, little endian.
.PP
Options specific to this program are:
.TP
.B \-b
.RS
Big endian transfer syntax.
.RE
.TP
.B \-e
.RS
Explicit value representation transfer syntax.
.RE
.SH ENVIRONMENT
.LP
\ 
.SH EXAMPLES
.LP
\n Sample attributes that can be fed to dc3create:
\n 
\n % dc3create >test
\n
\n (0000x8,0000x8) VR=<CS> VL=<0x14> <ORIGINAL\\PRIMARY\\MPR> 
\n (0000x8,000x16) VR=<UI> VL=<0x10> <1.2.840.10008.0> 
\n (000x28,0000x0) VR=<UL> VL=<0x4> [0xb6]
\n (000028,000002) VR=<US> VL=<2> [1]
\n (000028,000004) VR=<CS> VL=<c> <MONOCHROME2> 
\n (0x6000,000x50) VR=<SS> VL=<0x4> [0x5c31,0x2031]
.LP
\n Sample use of unspecified vl ...
\n 
\n % dc3create >test
\n
\n (0x7fe0,000x10) OX PixelData VR=<OW> VL=<0xffffffff> []
.LP
\n Sample use of data values ...
\n 
\n % dc3create >test
\n
\n (0x7fe0,000x10) OX PixelData VR=<OW> VL=<0x8> []
\n 0x10 0x12
\n 0x24 0x36
\n 
\n % dc3dump test
\n (0x7fe0,000x10) OX PixelData     VR=<OW>   VL=<0x8> 
\n  
\n % od -x test
\n 0000000  e07f 1000 0800 0000 1000 1200 2400 3600
\n 0000020
.LP
\n 
\n Sample use of metaheader, then explicit big syntax ...
\n 
\n % dc3create >test
\n
\n (0000x2,000x10) UI TransferSyntaxUID VR=<UI> VL=<0x14>
\n                 <1.2.840.10008.1.2.2>
\n %big
\n (000028,000002) VR=<US> VL=<2> [1]
.LP
\n Should be able to find no differences:
\n
\n % dc3dump test 2>test.dump
\n % dc3create <test.dump >test2
\n % cmp test test2
\n % dc3dump test2 2>test2.dump
\n % diff -b test.dump test2.dump
\n
(unless of course there are OB or OS attributes, in which case the data will be elided, and the comparison will fail, or there is a metaheader present which changes the transfer syntax in the middle of the file).
.SH FILES
.LP
\ 
.SH SEE ALSO
.BR dc3archive(1) ,
.BR dc3convert(1) ,
.BR dc3intro(1) ,
.BR dc3intro(3) ,
.BR dc3intro(5) ,
.BR dc3misc(1) ,
.BR dc3simple(1)
.SH AUTHOR
Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved.
.SH BUGS
In the attribute descriptions, the group, element, value length, and values are always in hexadecimal format, whether a leading "0x" is present or not. This is because dc3dump adopts such a convention, with slightly different output on different platforms and compilers. In particular lib-g++ does not output the "x" in hexadecimal formats :(

