static char *CopyrightIdentifier(void) { return "@(#)imgorder.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#ifdef __GNUG__
#include <builtin.h>
#endif
#include <fstream.h>
#include <stdlib.h>         // for atoi()
#include <ctype.h>

#include "GetOpt.h"
#include "MainCmd.h"

unsigned long
readBigEndianValue(istream& istr,unsigned nbytes)
{
	unsigned char *buffer;
	buffer=new unsigned char[nbytes];
	unsigned char *ptr = buffer;
	unsigned long value = 0;

	istr.read(buffer,nbytes);

	value=0;
	while (nbytes--) {
		value<<=8;
		value|=*ptr++;
	}
	if (buffer) delete[] buffer;
	return value;
}

unsigned long
swapByteOrder(unsigned long value,unsigned nbytes)
{
	unsigned long result=0;
	while (nbytes--) {
		result<<=8;
		result|=value&0xff;
		value>>=8;
	}
	return result;
}

unsigned long
changeSign(unsigned long value,unsigned nbytes)
{
	return value^(1<<(nbytes*8-1));
}


int
main(int argc,char **argv)
{
	CCOMMAND(argc,argv);

	// *************** Setup options ***************

	char *	inname = 0;

	unsigned databits	= 16;
	unsigned skipbytes	= 0;
	int verbose		= 0;

	GetOpt getopt(argc,argv,"d:j:s:v");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'd':	databits=atoi(getopt.optarg);
					break;
			case 'j':
			case 's':	skipbytes=atoi(getopt.optarg);
					break;
			case 'v':	verbose=1;
					break;
			case '?':
			default:
				err=1;
		}
	}

	istream *instream=&cin;

	if (!err && argc > getopt.optind) {
		if (argc <= getopt.optind+1) {
			inname=argv[getopt.optind];
			ifstream *infstream=new ifstream(inname,ios::in);
			if (!infstream->rdbuf()->is_open()) {
				cerr << "Can't open <" << inname
				     << "> for read\n" << flush;
				return 1;
			}
			else {
				instream=infstream;
			}
		}
		else err=1;
	}
	else {
		// default for some environments is unbuffered which is slow++
		// the following improves performance 25 fold with libg++ ...

		static const unsigned bufsize=4096;
		streambuf *in  = cin.rdbuf();
		static char inbuf[bufsize];
		in->setbuf(inbuf,sizeof(inbuf));
	}

	if (databits<8) databits*=8;	// assume specified as bytes

	if (databits%8) {
		cerr << "Error - data bits must be multiple of 8\n"
		     << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0]
		     << " [-d databits] [-j|s skipbytes] [-v]"
		     << " [infile]\n"
		     << flush;
		return 1;
	}

	// *************** Actually do something ***************

	// don't assume we can seek ...

	if (verbose) {
		cerr << "Skipping " << skipbytes << " bytes\n";
	}

	{ char buffer[1]; while (skipbytes--) instream->read(buffer,1); }

	if (!instream->good()) {
		cerr << "Error - couldn't jump over header\n" << flush;
		return 1;
	}

	unsigned databytes=databits/8;

	unsigned long min_ubig		= (unsigned long)((signed long)-1);
	unsigned long min_ulittle	= (unsigned long)((signed long)-1);
	unsigned long min_sbig		= (unsigned long)((signed long)-1);
	unsigned long min_slittle	= (unsigned long)((signed long)-1);
	unsigned long max_ubig		= 0;
	unsigned long max_ulittle	= 0;
	unsigned long max_sbig		= 0;
	unsigned long max_slittle	= 0;

	while (1) {
		unsigned long ubig = readBigEndianValue(*instream,databytes);
		if (!instream->good()) break;

		unsigned long ulittle	= swapByteOrder(ubig,databytes);
		signed   long slittle	= changeSign(ulittle,databytes);
		signed   long sbig	= changeSign(ubig,databytes);

		if (ubig < min_ubig) min_ubig=ubig;
		if (ubig > max_ubig) max_ubig=ubig;

		if (ulittle < min_ulittle) min_ulittle=ulittle;
		if (ulittle > max_ulittle) max_ulittle=ulittle;

		if (sbig < min_sbig) min_sbig=sbig;
		if (sbig > max_sbig) max_sbig=sbig;

		if (slittle < min_slittle) min_slittle=slittle;
		if (slittle > max_slittle) max_slittle=slittle;
	}

	unsigned long range_ubig	= max_ubig-min_ubig;
	unsigned long range_ulittle	= max_ulittle-min_ulittle;
	unsigned long range_sbig	= max_sbig-min_sbig;
	unsigned long range_slittle	= max_slittle-min_slittle;

	if (verbose) {
		cerr << "Unsigned Big-endian\t"
		     << " " << "min="   << min_ubig
		     << " " << "max="   << max_ubig
		     << " " << "range=" << range_ubig
		     << "\n";
		cerr << "Unsigned Little-endian\t"
		     << " " << "min="   << min_ulittle
		     << " " << "max="   << max_ulittle
		     << " " << "range=" << range_ulittle
		     << "\n";
		cerr << "Signed Big-endian\t"
		     << " " << "min="   << min_sbig
		     << " " << "max="   << max_sbig
		     << " " << "range=" << range_sbig
		     << "\n";
		cerr << "Signed Little-endian\t"
		     << " " << "min="   << min_slittle
		     << " " << "max="   << max_slittle
		     << " " << "range=" << range_slittle
		     << "\n";
	}

	if (databits == 8) {
		cout << "Single-byte ";
		if (range_ulittle < range_slittle) {
			cout << "Unsigned ";
		}
		else if (range_ulittle > range_slittle) {
			cout << "Signed ";
		}
		else {
			cout << "Sign unknown ";
		}
	}
	else if (range_ulittle < range_ubig || range_slittle < range_sbig) {
		cout << "Little-endian ";
		if (range_ulittle < range_slittle) {
			cout << "Unsigned ";
		}
		else if (range_ulittle > range_slittle) {
			cout << "Signed ";
		}
		else {
			if (min_ulittle < min_slittle) {
				cout << "Unsigned ";
			}
			else if (min_ulittle > min_slittle) {
				cout << "Signed ";
			}
			else {
				cout << "Sign unknown ";
			}
		}
	}
	else if (range_ulittle > range_ubig || range_slittle > range_sbig) {
		cout << "Big-endian ";
		if (range_ubig < range_sbig) {
			cout << "Unsigned ";
		}
		else if (range_ubig > range_sbig) {
			cout << "Signed ";
		}
		else {
			if (min_ubig < min_sbig) {
				cout << "Unsigned ";
			}
			else if (min_ubig > min_sbig) {
				cout << "Signed ";
			}
			else {
				cout << "Sign unknown ";
			}
		}
	}
	else {
		cout << "Byte-order unknown ";
	}
	cout << "\n";

	return 0;
}

