static char *CopyrightIdentifier(void) { return "@(#)imgpatch.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }


#include <fstream.h>
#include <stdlib.h>            // for atoi()
#include <string.h>            // for strlen()
#include <ctype.h>             // for isdigit()

#include "glist.h"

#include "GetOpt.h"
#include "MainCmd.h"

// In order for the sorted list template to know how to sort one needs
// to define a type that can order itself, hence this ...

class Patch {
	unsigned	Offset;
	unsigned	Length;
	char *		String;
	char *		PadString;
	char *		BinaryString;
	char *		BinaryPadString;

	char getEscape(char * & str)
		{
			unsigned char rval;
			++str;	// skip "\"
			switch (*str) {
				case 'r':
					rval='\r'; ++str; break;
				case 'n':
					rval='\n'; ++str; break;
				case 't':
					rval='\t'; ++str; break;
				case 'f':
					rval='\f'; ++str; break;
				case 'b':
					rval='\b'; ++str; break;
				default:	
					if (isdigit(*str)) {	// should be octal
						rval=0;
						while (isdigit(*str)) {
						 	rval=rval*8+
								(toascii(*str)-
								    toascii('0'));
							++str; // in case of macro
						}
					}
					else {
						rval=*str++; break;
					}
					break;
			}
			return (char)rval;
		}
public:
	Patch(unsigned offset,unsigned length,char *string,char *padstring)
		{
			Offset=offset;
			Length=length;
			String=string;
			PadString=padstring;
			BinaryString=new char[strlen(String)+1];
			char *from=String;
			char *to=BinaryString;
			while (*from) {
				if (*from == '\\') *to=getEscape(from);
				else *to=*from++;
				++to;
			}
			*to='\0';
			BinaryPadString=new char[strlen(PadString)+1];
			from=PadString;
			to=BinaryPadString;
			while (*from) {
				if (*from == '\\') *to=getEscape(from);
				else *to=*from++;
				++to;
			}
			*to='\0';
		}

	void		put(ostream& stream)
		{
			stream << dec << "@" << Offset
				      << "\t[" << Length
				      << "]\t<" << String
				      << ">\t<" << PadString
				      << ">";
		}

	unsigned	GetOffset(void)		{ return Offset; }
	unsigned	GetLength(void)		{ return Length; }
	char *		GetString(void)		{ return BinaryString; }
	char *		GetPadString(void)	{ return BinaryPadString; }
};
	

class PatchPointer {
	Patch *Pointer;
public:
	PatchPointer(void)		{ Pointer=0; }
	PatchPointer(Patch *p)		{ Pointer=p; }

	PatchPointer(const PatchPointer& t)
		{ Pointer=t.Pointer; }
	void operator=(const PatchPointer& t)
		{ Pointer=t.Pointer; }

	int operator==(const PatchPointer& p2)
		{ return Pointer->GetOffset() == p2.Pointer->GetOffset(); }
	int operator<(const PatchPointer& p2)
		{ return Pointer->GetOffset() <  p2.Pointer->GetOffset(); }
	int operator<=(const PatchPointer& p2)
		{ return Pointer->GetOffset() <= p2.Pointer->GetOffset(); }
	int operator>(const PatchPointer& p2)
		{ return Pointer->GetOffset() >  p2.Pointer->GetOffset(); }
	int operator>=(const PatchPointer& p2)
		{ return Pointer->GetOffset() >= p2.Pointer->GetOffset(); }

	Patch *	GetPointer()	{ return Pointer; }
};	

class PatchList : public GENSortedList<PatchPointer> {
public:
	int	insert(Patch *p)
		{
			PatchPointer ap(p);
			return GENSortedList<PatchPointer>::insert(ap);
		}

	Patch *	value(void)
		{
			return GENSortedList<PatchPointer>::
				value().GetPointer();
		}

	int	Check(void)
		{
			int bad=0;
			first();
			PatchPointer last = 0;
			while (ismore()) {
				int bothbad=0;
				PatchPointer current =
					GENSortedList<PatchPointer>::value();
				if (last.GetPointer() && last > current) {
					cerr << "Patches not in order ...\n";
					bothbad=1;
				}
				if (last.GetPointer() && last == current) {
					cerr << "Patches same offset ...\n";
					bothbad=1;
				}
				if (last.GetPointer()
				 &&  last.GetPointer()->GetOffset()
				   + last.GetPointer()->GetLength()
				   > current.GetPointer()->GetOffset()
				) {
					cerr << "Patches overlap ...\n";
					bothbad=1;
				}
				if (bothbad) {
					last.GetPointer()->put(cerr);
					cerr << " cf. ";
					current.GetPointer()->put(cerr);
					cerr << "\n" << flush;
					bad=1;
				}
				int currentbad=0;
				if (current.GetPointer()->GetLength() <
				    strlen(current.GetPointer()->GetString())) {
					cerr << "Patch string too long ...\n";
					currentbad=1;
				}
				if (currentbad) {
					current.GetPointer()->put(cerr);
					cerr << "\n" << flush;
					bad=1;
				}
				last=current;
				next();
			}
			return bad;
		}
};

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	char *	inname = 0;
	char *	outname = 0;
	int 	verbose = 0;

	GetOpt getopt(argc,argv,"r[4]v");
	int c;
	int err=0;
	PatchList patchlist;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'v':	// Verbose
				verbose=1;
				break;
			case 'r':	// Patch offset,length,string,pad
				{
					unsigned offset=atoi(getopt.optarg);
					unsigned
					    length=atoi(argv[getopt.optind]);
					char *string=argv[getopt.optind+1];
					char *pad=argv[getopt.optind+2];
					Patch *patch = new
					    Patch(offset,length,string,pad);
					patchlist.insert(patch);
				}
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind+2 == argc) {
		inname=argv[getopt.optind];
		outname=argv[getopt.optind+1];
	}
	else err=1;

	if (err) {
		cerr << "Usage: " << argv[0]
		     << " -r offset length string padvalue infile outfile\n"
		     << flush;
		return 1;
	}

	if (verbose) {
		cerr << "************** Replacements ************\n" << flush;
		patchlist.first();
		while (patchlist.ismore()) {
			Patch *patch = patchlist.value();
			patch->put(cerr);
			cerr << "\n" << flush;
			patchlist.next();
		}
	}

	if (patchlist.Check()) {
		cerr << "Some patches are bad, so won't do any.\n" << flush;
		return 1;
	}

	ifstream instream(inname,ios::in);
	if (!instream.rdbuf()->is_open()) {
		cerr << "Can't open <" << inname << "> for read\n" << flush;
		return 1;
	}

	ofstream outstream(outname,ios::out);
	if (!outstream.rdbuf()->is_open()) {
		cerr << "Can't open <" << outname << "> for write\n" << flush;
		return 1;
	}

	patchlist.first();
	Patch *patch;
	if (patchlist.ismore())
		patch=patchlist.value();
	else
		patch=0;
	unsigned offset=0;
	while (1) {
		char c;
		instream.read(&c,1);
		if (instream.eof() || !instream.good()) break;
		if (patch
		 && offset >= patch->GetOffset()
		 && offset <  patch->GetOffset()+patch->GetLength()) {
			unsigned index=offset-patch->GetOffset();
			if (index < strlen(patch->GetString())) {
				outstream.write(patch->GetString()+index,1);
			}
			else {
				outstream.write(patch->GetPadString(),1);
			}
			if (++offset >= patch->GetOffset()+patch->GetLength()) {
				patchlist.next();
				if (patchlist.ismore()) {
					patch=patchlist.value();
				}
				else {
					patch=0;
				}
			}
		}
		else {
			outstream.write(&c,1);
			++offset;
		}
	}

	outstream.close();
	return 0;
}

