static char *CopyrightIdentifier(void) { return "@(#)pgmdiff.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <fstream.h>
// #include <iomanip.h>
#include <strstream.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "GetOpt.h"
#include "MainCmd.h"

static
char
pnm_getc(istream &istr)		// 1st non-comment, non-whitespace, 0 on failure
{
	char c;
	while (istr.get(c) && (c == ' ' || c == '\t' || c == '\n'));
	if (!istr) return 0;
	if (c == '#') {		// skip comments
		while (istr.get(c) && c != '\n');
		if (!istr) return 0;
		while (istr.get(c) && (c == ' ' || c == '\t' || c == '\n'));
		if (!istr) return 0;
	}
	return c;
}

static
int
pnm_getu(istream &istr)		// get unsigned value, -1 on failure
{
	char c=pnm_getc(istr);	// skips comments and whitespace
	if(!c || !isdigit(c)) return -1;
	unsigned value=0;
	while (c && isdigit(c)) {
		value*=10;
		value+=(c-'0');
		istr.get(c);
	}
	// discards single trailing white space
	return value;
}

static
unsigned short
pnm_geti(istream &istr,unsigned short wordlength)	// get image data value
{
	unsigned char c[2];
	istr.read(c,wordlength);
	unsigned short value;
	// PNM format is little-endian ...
	if (wordlength == 2)
		value=(c[1]<<8)+c[0];
	else
		value=c[0];
	return value;
}

static
void
pnm_puti(ostream &ostr,unsigned short value,unsigned short wordlength)
							// put image data value
{
	unsigned char c[2];
	// PNM format is little-endian ...
	if (wordlength == 2) {
		c[0]=value&0xff;
		c[1]=value>>8;
	}
	else
		c[0]=value;
	ostr.write(c,wordlength);
}

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	cerr.setf(ios::showbase,ios::showbase);

	int		verbose = 0;
	char		useoffset = 0;
	char		rangecheck = 0;
	unsigned 	outputbits	= 0;
	unsigned 	addvalue	= 0;

	GetOpt getopt(argc,argv,"a:d:orv");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'a':
				addvalue=atoi(getopt.optarg);
				break;
			case 'd':
				outputbits=atoi(getopt.optarg);
				break;
			case 'o':
				useoffset=1;
				break;
			case 'r':
				rangecheck=1;
				break;
			case 'v':
				verbose=1;
				break;
			case '?':
			default:
				err=1;
				break;
		}
	}

	unsigned nfiles=argc-getopt.optind;
	char **filenames;
	if (nfiles != 2) {
		cerr << "Must specify two filenames as input\n" << flush;
		err=1;
	}
	else {
		filenames=&argv[getopt.optind];
	}

	if (outputbits && outputbits<8) outputbits*=8;

	if (outputbits && outputbits%8) {
		cerr << "Error - data length must be 1,2 or multiple of 8\n"
		     << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0]
		     << " [-d outputbits][-o][-r][-v] filea fileb\n"
		     << flush;
		return 1;
	}

	char *filenamea = filenames[0];
	char *filenameb = filenames[1];

	ifstream *ina = new ifstream(filenamea);
	if (!ina->rdbuf()->is_open()) {
		cerr << "Can't open " << filenamea << "\n" << flush;
		return 1;
	}
	ifstream *inb = new ifstream(filenameb);
	if (!inb->rdbuf()->is_open()) {
		cerr << "Can't open " << filenameb << "\n" << flush;
		return 1;
	}

	// Read PNM headers ...

	char magica=pnm_getc(*ina);
	char formata;
	ina->get(formata);

	if (!ina->good() || magica != 'P') {
		cerr << "Not a pnm file " << filenamea << "\n" << flush;
		return 1;
	}

	if (formata != '5') {
		cerr << "Unsupported pnm format P" << formata
		     << " in " << filenamea
		     << "\n" << flush;
		return 1;
	}

	int widtha = pnm_getu(*ina);
	int heighta = pnm_getu(*ina);
	int maxvala = pnm_getu(*ina);
	unsigned maxvallengtha=(maxvala <= 255) ? 1 : 2;

	if (!*ina || widtha == -1 || heighta == -1 || maxvala == -1) {
		cerr << "Corrupt pnm header " << filenamea << "\n" << flush;
		return 1;
	}

	char magicb=pnm_getc(*inb);
	char formatb;
	inb->get(formatb);

	if (!inb->good() || magicb != 'P') {
		cerr << "Not a pnm file " << filenameb << "\n" << flush;
		return 1;
	}

	if (formatb != '5') {
		cerr << "Unsupported pnm format P" << formatb
		     << " in " << filenameb
		     << "\n" << flush;
		return 1;
	}

	int widthb = pnm_getu(*inb);
	int heightb = pnm_getu(*inb);
	int maxvalb = pnm_getu(*inb);
	unsigned maxvallengthb=(maxvalb <= 255) ? 1 : 2;

	if (!*inb || widthb == -1 || heightb == -1 || maxvalb == -1) {
		cerr << "Corrupt pnm header " << filenameb << "\n" << flush;
		return 1;
	}

	if (verbose) {
		cerr << "File " << filenamea << "\n" << flush;
		cerr << "\twidth " << widtha << "\n" << flush;
		cerr << "\theight " << heighta << "\n" << flush;
		cerr << "\tmaxval " << maxvala << "\n" << flush;
		cerr << "\tbytes per pixel " << maxvallengtha << "\n" << flush;
		cerr << "File " << filenameb << "\n" << flush;
		cerr << "\twidth " << widthb << "\n" << flush;
		cerr << "\theight " << heightb << "\n" << flush;
		cerr << "\tmaxval " << maxvalb << "\n" << flush;
		cerr << "\tbytes per pixel " << maxvallengthb << "\n" << flush;
	}

	if (formata != formatb
	 || widtha != widthb
	 || heighta != heightb
	 ) {
		cerr << "PNM files different formats or sizes :(\n" << flush;
		return 1;
	}

	// Last whitespace discarded ... at start of data

	int width  = widtha;
	int height = heighta;
	unsigned short wordlength;
	if (outputbits) 
	 	wordlength=(outputbits-1)/8u+1;
	else
	 	wordlength=(maxvallengtha>maxvallengthb) ? maxvallengtha : maxvallengthb;

	int maxval = wordlength == 1 ? 0xff : 0xffff;

	if (verbose) {
		cerr << "Output\n" << flush;
		cerr << "\twidth " << width << "\n" << flush;
		cerr << "\theight " << height << "\n" << flush;
		cerr << "\tmaxval " << maxval << "\n" << flush;
		cerr << "\tbytes per pixel " << wordlength << "\n" << flush;
	}

	// Write new PNM header ...

	{
		ostrstream ost;
		ost << "P5\n" << width << " " << height << "\n"
		    << maxval << "\n" << '\0';
		char *str=ost.str();
		cout.write(str,strlen(str));
		delete str;
	}

	// Process image data ...

	// NB. Goal is to create a difference image used as:
	//
	//	process <orig >new
	//	pgmdiff new orig > diff
	//	pgmdiff new diff > orig.2
	//	cmp orig orig.2
	//
	// yields no difference (ie. new-(new-orig) == orig)
	//
	// hopefully will work for both signed and unsigned values
	//
	// if -o is specified then an offset will be added to shift
	// the signed difference value to the middle of an unsigned
	// range ...

	unsigned y;
	unsigned short mask=(wordlength==2)?0xffff:0xff;
	long largestplus =(wordlength==2)?(0x7fff):(0x7f);
	long largestminus=(-1l)^largestplus;
	if (verbose) {
		cerr << "Largest +ve in output "
		     << dec << (unsigned long)largestplus
		     << " " << dec << largestplus
		     << " 0x" << hex << (unsigned long)largestplus
		     << "\n";
		cerr << "Largest -ve in output "
		     << dec << (unsigned long)largestminus
		     << " " << dec << largestminus
		     << " 0x" << hex << (unsigned long)largestminus
		     << "\n";
	}
	unsigned short offset=((useoffset) ? ((wordlength==2)?0x8000:0x80) : 0) + addvalue;
	if (verbose) {
		cerr << "Offset "
		     << dec << offset
		     << " " << dec << (signed short)offset
		     << " 0x" << hex << offset
		     << "\n";
	}
	long mindiff=largestplus;
	long maxdiff=largestminus;
	for (y=0; y<height; ++y) {
		unsigned x;
		for (x=0; x<width; ++x) {
			unsigned short valuea=pnm_geti(*ina,wordlength);
			unsigned short valueb=pnm_geti(*inb,wordlength);

			long value = (long)valuea-(long)valueb+(long)offset;
			if (rangecheck) {
				if (value > largestplus+offset || value < largestminus+offset) {
					cerr << dec << "At row " << y << " col " << x
					     << " (output byte " << (x+y*width)*wordlength
					     << ") difference value out of range "
					     << valuea << "-" << valueb << "=" << value
					     << "\n";
					return 1;
				}
			}

			unsigned short ovalue=(unsigned short)(value)&mask;
			pnm_puti(cout,ovalue,wordlength);

			if (value < mindiff) mindiff=value;
			if (value > maxdiff) maxdiff=value;
		}
	}

	if (verbose) {
		cerr << "Minimum output value "
		     << dec << mindiff
		     << " " << ((wordlength==2)
				? (signed short)mindiff
				: (signed short)((signed char)mindiff)
				)
		     << " 0x" << hex << mindiff
		     << "\n" << flush;
		cerr << "Maximum output value "
		     << dec << maxdiff
		     << " " << ((wordlength==2)
				? (signed short)maxdiff
				: (signed short)((signed char)maxdiff)
				)
		     << " 0x" << hex << maxdiff
		     << "\n" << flush;
	}

	if (!ina->good()) {
		cerr << "Read error on " << filenamea << "\n" << flush;
		return 1;
	}
	if (!inb->good()) {
		cerr << "Read error on " << filenameb << "\n" << flush;
		return 1;
	}
	if (!cout.good()) {
		cerr << "Write error\n" << flush;
		return 1;
	}

	return 0;
}

