static char *CopyrightIdentifier(void) { return "@(#)pgmtowrw.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <iostream.h>
#include <ctype.h>

#include "GetOpt.h"
#include "MainCmd.h"

static
char
pnm_getc(istream &istr)		// 1st non-comment, non-whitespace, 0 on failure
{
	char c;
	while (istr.get(c) && (c == ' ' || c == '\t' || c == '\n'));
	if (!istr) return 0;
	if (c == '#') {		// skip comments
		while (istr.get(c) && c != '\n');
		if (!istr) return 0;
		while (istr.get(c) && (c == ' ' || c == '\t' || c == '\n'));
		if (!istr) return 0;
	}
	return c;
}

static
int
pnm_getu(istream &istr)		// get unsigned value, -1 on failure
{
	char c=pnm_getc(istr);	// skips comments and whitespace
	if(!c || !isdigit(c)) return -1;
	unsigned value=0;
	while (c && isdigit(c)) {
		value*=10;
		value+=(c-'0');
		istr.get(c);
	}
	// discards single trailing white space
	return value;
}

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	cerr.setf(ios::showbase,ios::showbase);

	char		endian = 0;
	int		verbose = 0;

	GetOpt getopt(argc,argv,"lbv");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'l':	// little endian
			case 'b':	// big endian
				endian=c;
				break;
			case 'v':
				verbose=1;
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind != argc) {
		cerr << "Trailing garbage after options\n" << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0] << " -l|b -v\n"
		     << flush;
		return 1;
	}

	char magic=pnm_getc(cin);
	char format;
	cin.get(format);

	if (!cin || magic != 'P') {
		cerr << "Not pnm file\n" << flush;
		return 1;
	}

	if (format != '5') {
		cerr << "Unsupported pnm format P" << format << "\n" << flush;
		return 1;
	}

	int width = pnm_getu(cin);
	int height = pnm_getu(cin);
	int maxval = pnm_getu(cin);

	if (verbose) {
		cerr << "width " << width << "\n" << flush;
		cerr << "height " << height << "\n" << flush;
		cerr << "maxval " << maxval << "\n" << flush;
	}

	if (!cin || width == -1 || height == -1 || maxval == -1) {
		cerr << "Corrupt pnm header\n" << flush;
		return 1;
	}

	// Last whitespace discarded ... at start of data

	unsigned wordlength;
	if (maxval <= 255) {
		wordlength=1;
	}
	else {
		if (!endian) {
			cerr << "Must specify byte order\n" << flush;
			return 1;
		}
		wordlength=2;
	}

	// default for some environments is unbuffered which is slow++
	// the following improves performance 25 fold with libg++ ...

	const unsigned bufsize=4096;
	streambuf *in  = cin.rdbuf();
	streambuf *out = cout.rdbuf();
	static char inbuf[bufsize];
	static char outbuf[bufsize];
	in->setbuf(inbuf,sizeof(inbuf));
	out->setbuf(outbuf,sizeof(outbuf));

	// optimize for cases where byte swap unnecessary
	// (improves performance 5 to 7 fold)

	unsigned i;
	if (wordlength == 1 || endian == 'l') {	// PNM is little-endian
		unsigned long chunks=(unsigned long)
					width*height*wordlength/bufsize;
		unsigned long left=(unsigned long)
					width*height*wordlength%bufsize;
		static char buf[bufsize];
		for (i=0; i<chunks; ++i) {
			unsigned n=in->sgetn(buf,bufsize);
			out->sputn(buf,n);
		}
		if (left) {
			unsigned n=in->sgetn(buf,(int)left);
			out->sputn(buf,n);
		}
	}
	else {
		for (i=0; i<width*height; ++i) {
			char c[2];
			cin.read(c,wordlength);
			// endian == 'b'
			cout.write(&c[1],1);
			cout.write(&c[0],1);
		}
	}

	if (!cin.good()) {
		cerr << "Read error\n" << flush;
		return 1;
	}
	if (!cout.good()) {
		cerr << "Write error\n" << flush;
		return 1;
	}

	return 0;
}

