static char *CopyrightIdentifier(void) { return "@(#)rawxbrws.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <stdlib.h>		// for atoi()
#include <fstream.h>

#define INSTANTIATEGLOBALS

#include "gnerror.h"
#include "vgdisplay.h"

#include "GetOpt.h"
#include "MainCmd.h"

#define MAXDISPLAYEDWIDTH 1024

static void
reloadimages(unsigned nfiles,
	VGWindowImage *wimage,VGImage **images,VGColormap *map)
{
	unsigned xposn,yposn,i;
	for (xposn=0,yposn=0,i=0;i<nfiles;i++) {
		wimage->Load(images[i],map,xposn,yposn);
		if ((xposn+=images[i]->getHeight()) >= wimage->getWidth()) {
			xposn=0;
			yposn+=images[i]->getHeight();
		}
	}
}

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	cerr.setf(ios::showbase,ios::showbase);

	char		endian = 0;
	unsigned	width = 0;
	unsigned	height = 0;
	unsigned	depth = 8;
	char		windowing = 'f';
	unsigned	usecin = 0;

	GetOpt getopt(argc,argv,"lbw:h:d:fs");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case '-':	// use standard input
				usecin=1;
				break;
			case 'l':	// little endian
			case 'b':	// big endian
				endian=c;
				break;
			case 'w':	// width
				width=atoi(getopt.optarg);
				break;
			case 'h':	// height
				height=atoi(getopt.optarg);
				break;
			case 'd':	// depth - defaults to 8
				depth=atoi(getopt.optarg);
				break;
			case 's':	// slow - image windowing
			case 'f':	// fast - map windowing
				windowing=c;
				break;
			case '?':
			default:
				cerr << "Illegal option <" << (char)c 
				     << ">\n" << flush;
				err=1;
		}
	}

	if (usecin && getopt.optind != argc) {
		cerr << "Can't have filename list if reading standard input\n"
		     << flush;
		err=1;
	}

	switch (depth) {
		case 8:
			if (endian) {
				cerr << "Can't have byte order for 8 bits !\n"
				     << flush;
				err=1;
			}
			break;
		case 12:
		case 16:
			if (!endian) {
				cerr << "Must specify byte order for "
				     << depth << " bit data\n"
				     << flush;
				err=1;
			}
			break;
		default:
			cerr << "Unsupported data length " << depth << "\n"
			     << flush;
			err=1;
			break;
	}

	if (!width || !height) {
		cerr << "Must specify width & height\n" << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0]
		     << " -w nn -h nn [-d nn -l|b] [-f|s]"
		     << " [-|filenames]\n" << flush;
		return 1;
	}

	unsigned nfiles=argc-getopt.optind;
	char **filenames;
	if (usecin || nfiles == 0) {
		usecin=1;
		filenames=0;
		nfiles=1;
	}
	else {
		filenames=&argv[getopt.optind];
	}

	{
		unsigned imageswide=MAXDISPLAYEDWIDTH/width;
		if (imageswide > nfiles) imageswide=nfiles;
		unsigned wimagewidth=width*imageswide;
		unsigned imageshigh=(nfiles-1)/imageswide+1;
		unsigned wimageheight=height*imageshigh;

		VGDisplay display;
		VGWindow window(&display,wimagewidth,wimageheight);

		VGWindowImage wimage(&window);

		VGColormap map(&display);
		VGColormap tmpmap(&display);

		VGImage **images = new VGImage * [nfiles];

		if (!images) {
			cerr << "Can't allocate images array\n" << flush;
			return 1;
		}

		switch (depth) {
			case 8:
			case 12:
			case 16:
				map.setLinearGray(1<<depth);
				if (windowing == 'f')
					tmpmap.setLinearGray(1<<depth);
				break;
			default:
				cerr << "Unimplemented depth = "
				     << depth << "\n" << flush;
				return 1;
		}

		unsigned i;
		for (i=0;i<nfiles;i++) {
			istream *instream;
			if (usecin) {
				instream=&cin;
			}
			else {
				char *name=*filenames++;
				ifstream *infstream = new ifstream(name);
				if (!infstream->rdbuf()->is_open()) {
					cerr << "Can't open " 
					     << name << "\n" << flush;
					return 1;
				}
				instream=infstream;
			}
			VGImage *image;
			switch (depth) {
				case 8:
					image=new VGImage_8bit(width,height);
					image->Read(instream);
					break;
				case 12:
					image=new VGImage_12bit(width,height);
					image->Read(instream,endian);
					break;
				case 16:
					image=new VGImage_16bit(width,height);
					image->Read(instream,endian);
					break;
			}
			images[i]=image;
			if (!usecin) ((ifstream *)instream)->close();
		}

		int maplevel=1054;
		int mapwidth=80;
		if (windowing == 's') {
			map.setImageLevelWidth(maplevel,mapwidth);
			window.setColormap(&map);
		}
		else {
			tmpmap.setMapLevelWidth(maplevel,mapwidth);
			window.setColormap(&tmpmap);
		}
		reloadimages(nfiles,&wimage,images,&map);
		window.SelectInput();
		window.Map();

		while (1) {
			XEvent report = display.NextEvent();
			switch (report.type) {
				int lastx,lasty;
				case Expose:
					if (report.xexpose.count != 0) break;
					wimage.Put();
					break;
				case ButtonPress:
					if (report.xbutton.button == Button1) {
						lastx=report.xbutton.x;
						lasty=report.xbutton.y;
					}
					break;
				case ButtonRelease:
					// up=increase level,down=decrease
					// left=narrow window,right=widen
					if (report.xbutton.button == Button1) {
						int deltax=(int)
							report.xbutton.x-lastx;
						int deltay=(int)
							report.xbutton.y-lasty;
						cerr << "delta "
						     << deltax << ","
						     << deltay << "\n" << flush;
						mapwidth+=deltax;
						if (mapwidth<0) mapwidth=0;
						maplevel-=deltay;
						if (maplevel<0) maplevel=0;
						cerr << "width " << mapwidth 
						     << " level " << maplevel 
						     << "\n" << flush;
						if (windowing == 's') {
							map.setImageLevelWidth(
								maplevel,
								mapwidth);
							reloadimages(nfiles,
								&wimage,
								images,&map);
							wimage.Put();
						}
						else {
							tmpmap.setMapLevelWidth(
								maplevel,
								mapwidth);
							window.setColormap(
								&tmpmap);
						}
					}
					break;
				default:
					break;
			}
		}
	}

	return 0;
}

