static char *CopyrightIdentifier(void) { return "@(#)rawxdisp.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }


#include <stdlib.h>		// for atoi()

#define INSTANTIATEGLOBALS

#include "gnerror.h"
#include "vgdisplay.h"

#include "GetOpt.h"
#include "MainCmd.h"

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	cerr.setf(ios::showbase,ios::showbase);

	char		endian = 0;
	unsigned	width = 0;
	unsigned	height = 0;
	unsigned	depth = 8;

	GetOpt getopt(argc,argv,"lbw:h:d:");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'l':	// little endian
			case 'b':	// big endian
				endian=c;
				break;
			case 'w':	// width
				width=atoi(getopt.optarg);
				break;
			case 'h':	// height
				height=atoi(getopt.optarg);
				break;
			case 'd':	// depth - defaults to 8
				depth=atoi(getopt.optarg);
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind != argc) {
		cerr << "Trailing garbage after options\n" << flush;
		err=1;
	}

	switch (depth) {
		case 8:
			if (endian) {
				cerr << "Can't have byte order for 8 bits !\n"
				     << flush;
				err=1;
			}
			break;
		case 12:
		case 16:
			if (!endian) {
				cerr << "Must specify byte order for "
				     << depth << " bit data\n"
				     << flush;
				err=1;
			}
			break;
		default:
			cerr << "Unsupported data length " << depth << "\n"
			     << flush;
			err=1;
			break;
	}

	if (!width || !height) {
		cerr << "Must specify width & height\n" << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0] << " -w nn -h nn [-d nn -l|b]\n"
		     << flush;
		return 1;
	}

	{
		VGDisplay display;
		VGWindow window(&display,width,height);

		VGWindowImage wimage(&window);
		VGColormap map(&display);

		VGImage *image = 0;

		switch (depth) {
			case 8:
				image = new VGImage_8bit (width,height);
				map.setLinearGray(1<<8);
				image->Read(&cin);
				break;
			case 12:
				image = new VGImage_12bit (width,height);
				map.setLinearGray(1<<12);
				image->Read(&cin,endian);
				break;
			case 16:
				image = new VGImage_16bit (width,height);
				map.setLinearGray(1<<16);
				image->Read(&cin,endian);
				break;
			default:
				cerr << "Unimplemented depth = "
				     << depth << "\n" << flush;
				return 1;
		}

		wimage.Load(image,&map);
		window.setColormap(&map);

		window.SelectInput();
		window.Map();

		while (1) {
			XEvent report = display.NextEvent();
			switch (report.type) {
				case Expose:
					if (report.xexpose.count != 0) break;
					wimage.Put();
					break;
				default:
					break;
			}
		}
	}

	return 0;
}

