static char *CopyrightIdentifier(void) { return "@(#)wrwwind.cc 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved."; }

#include <stdlib.h>		// for atoi()
#include <iostream.h>
#include <strstream.h>
#include <string.h>

#include "GetOpt.h"
#include "MainCmd.h"

int
main(int argc, char *argv[])
{
	CCOMMAND(argc,argv);

	cerr.setf(ios::showbase,ios::showbase);

	char		endian = 0;
	unsigned	imagewidth = 0;
	unsigned	imageheight = 0;
	unsigned	depth = 8;

	unsigned	windowcentre = 0;
	unsigned	windowwidth = 0;
	float		slopescale = 1.0;
	float		slopeintercept = 0.0;
	unsigned	magnitudeoffset = 0;

	unsigned	wordlength = 0;

	GetOpt getopt(argc,argv,"lbw:h:d:c:t:s:i:o:a");
	int c;
	int err=0;
	while ((c=getopt()) != -1 && !err) {
		switch (c) {
			case 'l':	// little endian
			case 'b':	// big endian
				endian=c;
				break;
			case 'w':	// imagewidth
				imagewidth=atoi(getopt.optarg);
				break;
			case 'h':	// imageheight
				imageheight=atoi(getopt.optarg);
				break;
			case 'd':	// depth - defaults to 8
				depth=atoi(getopt.optarg);
				break;
			case 'c':	// windowcentre
				windowcentre=atoi(getopt.optarg);
				break;
			case 't':	// windowwidth
				windowwidth=atoi(getopt.optarg);
				break;
			case 's':	// slopescale
				slopescale=atof(getopt.optarg);
				break;
			case 'i':	// slopeintercept
				slopeintercept=atof(getopt.optarg);
				break;
			case 'o':	// magnitudeoffset
				magnitudeoffset=atoi(getopt.optarg);
				break;
			case 'a':	// CAT - use Hounsfield units
				magnitudeoffset=1024;
				break;
			case '?':
			default:
				err=1;
		}
	}

	if (getopt.optind != argc) {
		cerr << "Trailing garbage after options\n" << flush;
		err=1;
	}

	switch (depth) {
		case 1:
		case 8:
			if (endian) {
				cerr << "Can't have byte order for 8 bits !\n"
				     << flush;
				err=1;
			}
			wordlength=1;
			break;
		case 2:
		case 12:
		case 16:
			if (!endian) {
				cerr << "Must specify byte order for "
				     << depth << " bit data\n"
				     << flush;
				err=1;
			}
			wordlength=2;
			break;
		default:
			cerr << "Unsupported data length " << depth << "\n"
			     << flush;
			err=1;
			break;
	}

	if (!imagewidth || !imageheight) {
		cerr << "Must specify imagewidth & imageheight\n" << flush;
		err=1;
	}

	if (err) {
		cerr << "Usage: " << argv[0] << " -w nn -h nn [-d nn -l|b]"
		     << "[-c windowcenter -t windowwidth]"
		     << "[-o magnitudeoffset|-a]"
		     << "\n" << flush;
		return 1;
	}

	unsigned i;
	for (i=0; i<imagewidth*imageheight; ++i) {
		unsigned char c1,c2;
		cin.read(&c1,1);
		long word;
		if (wordlength == 1) {
			word=c1;
		}
		else {
			cin.read(&c2,1);
			word=(endian == 'l') ? (c2<<8)+c1 : (c1<<8)+c2;
		}
		word=word-magnitudeoffset-windowcentre+windowwidth/2;
		if (word < 0) word=0;
		if (word > windowwidth-1) word=windowwidth-1;
		word=(word*255)/(windowwidth-1);
		if      (word < 0)   word=0;
		else if (word > 255) word=255;
		c1=(unsigned char)word;
		cout.write(&c1,1);
	}
	if (!cin.good()) {
		cerr << "Read error\n" << flush;
		return 1;
	}
	if (!cout.good()) {
		cerr << "Write error\n" << flush;
		return 1;
	}

	return 0;
}

