/* syttype.h 0.12 95/09/18 Copyright (C) 1993,1994,1995. David A. Clunie. All rights reserved. */

#ifndef _DC3syttype_HEADER_
#define _DC3syttype_HEADER_

#include <math.h>
#include <string.h>
#include <stdlib.h>	// for atoi()

#include <fstream.h>

#include "ttypes.h"

/* ************************ Sytec Classes ************************ */

class SYTEC_float32 {
	double		value;

	char		*sh;
	char		*toString(void);
public:
	SYTEC_float32(unsigned l)
		{
			if (l != 1) {
				cerr << "SYTEC:: Only length 1 supported, not "
				     << l << "\n" << flush;
			}
			sh=0;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toDS(void)	{ return toString(); }
	double	todouble(void)	{ return value; }
};

class SYTEC_int16 {
	Int16	u;
	char	*sh;
	char	*toString(void);
public:
	SYTEC_int16(unsigned l)
		{
			if (l != 1) {
				cerr << "SYTEC:: Only length 1 supported, not "
				     << l << "\n" << flush;
			}
			sh=0;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toSH(void)	{ return toString(); }
	char *	toIS(void)	{ return toString(); }
	char *	toDS(void)	{ return toString(); }
	char *	toLO(void)	{ return toString(); }

	Uint16	toUS(void)	{ return (Uint16)u; }
	Uint32	toUL(void)	{ return (Uint32)u; }
	Int16	toSS(void)	{ return (Int16)u; }
	Int32	toSL(void)	{ return (Int32)u; }
	Uint16	toXS(void)	{ return (Uint16)u; }
};

class SYTEC_u_int16 {
	Uint16	u;
	char	*sh;
	char	*toString(void);
public:
	SYTEC_u_int16(unsigned l)
		{
			if (l != 1) {
				cerr << "SYTEC:: Only length 1 supported, not "
				     << l << "\n" << flush;
			}
			sh=0;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toSH(void)	{ return toString(); }
	char *	toIS(void)	{ return toString(); }
	char *	toDS(void)	{ return toString(); }
	char *	toLO(void)	{ return toString(); }

	Uint16	toUS(void)	{ return (Uint16)u; }
	Uint32	toUL(void)	{ return (Uint32)u; }
	Int16	toSS(void)	{ return (Int16)u; }
	Int32	toSL(void)	{ return (Int32)u; }
	Uint16	toXS(void)	{ return (Uint16)u; }
};

class SYTEC_int32 {
	Int32	u;
	char	*sh;
	char	*toString(void);
public:
	SYTEC_int32(unsigned l)
		{
			if (l != 1) {
				cerr << "SYTEC:: Only length 1 supported, not "
				     << l << "\n" << flush;
			}
			sh=0;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toSH(void)	{ return toString(); }
	char *	toIS(void)	{ return toString(); }
	char *	toDS(void)	{ return toString(); }
	char *	toLO(void)	{ return toString(); }

	Uint16	toUS(void)	{ return (Uint16)u; }
	Uint32	toUL(void)	{ return (Uint32)u; }
	Int16	toSS(void)	{ return (Int16)u; }
	Int32	toSL(void)	{ return (Int32)u; }
	Uint16	toXS(void)	{ return (Uint16)u; }
};

class SYTEC_u_int32 {
	Uint32	u;
	char	*sh;
	char	*toString(void);
public:
	SYTEC_u_int32(unsigned l)
		{
			if (l != 1) {
				cerr << "SYTEC:: Only length 1 supported, not "
				     << l << "\n" << flush;
			}
			sh=0;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toSH(void)	{ return toString(); }
	char *	toIS(void)	{ return toString(); }
	char *	toDS(void)	{ return toString(); }
	char *	toLO(void)	{ return toString(); }

	Uint16	toUS(void)	{ return (Uint16)u; }
	Uint32	toUL(void)	{ return (Uint32)u; }
	Int16	toSS(void)	{ return (Int16)u; }
	Int32	toSL(void)	{ return (Int32)u; }
	Uint16	toXS(void)	{ return (Uint16)u; }
};

class SYTEC_string {
	char *str;
	unsigned inlength;	// inlength is in bytes
public:
	SYTEC_string(unsigned l)
		{
			str=new char[l+1];
			if (str) {
				inlength=l;
				*str='\0';
			}
			else {
				inlength=0;
			}
		}

	~SYTEC_string(void)
		{
			if (str) delete[] str;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toLO(void)	{ return str; }
	char *	toPN(void)	{ return str; }
	char *	toSH(void)	{ return str; }
	char *	toCS(void)	{ return str; }
	char *	toLT(void)	{ return str; }
	char *	toIS(void)	{ return str; }
	char *	toDS(void)	{ return str; }
	Uint16	toUS(void)	{ return atoi(str); }
};

class SYTEC_byte {
	Uint16	u;
	char	*sh;
	char	*toString(void);
public:
	SYTEC_byte(unsigned l)
		{
			if (l != 1) {
				cerr << "SYTEC:: Only length 1 supported, not "
				     << l << "\n" << flush;
			}
			sh=0;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toSH(void)	{ return toString(); }
	char *	toIS(void)	{ return toString(); }
	char *	toDS(void)	{ return toString(); }
	char *	toLO(void)	{ return toString(); }

	Uint16	toUS(void)	{ return (Uint16)u; }
	Uint32	toUL(void)	{ return (Uint32)u; }
	Int16	toSS(void)	{ return (Int16)u; }
	Int32	toSL(void)	{ return (Int32)u; }
	Uint16	toXS(void)	{ return (Uint16)u; }
};

class SYTEC_datetime {
	Uint32	SecondsSinceEpoch;
	unsigned long	Year;
	unsigned long	Month;
	unsigned long	Day;
	unsigned long	Hours;
	unsigned long	Minutes;
	unsigned long	Seconds;
	char *dash;
	char *tmsh;
	char *dtsh;
public:
	SYTEC_datetime(unsigned l)
		{
			if (l != 1) {
				cerr << "SYTEC:: Only length 1 supported, not "
				     << l << "\n" << flush;
			}
			dash=0;
			tmsh=0;
			dtsh=0;
		}

	void	read(ifstream& instream,unsigned block,unsigned offset);

	char *	toDA(void);
	char *	toTM(void);
	char *	toDT(void);
};

#endif /* _DC3syttype_HEADER_ */
