#!/bin/sh

# testconv converter topnm dumper dicomcopier dicomdumper lengthawk testfiles

# effectively tests (with all implemented dicom transfer syntaxes):
#
#	- converter
#	- simple (topnm) convertor
#	- the dumper
#	- the dicom copier
#	- the dicom dumper
#	- the dicom creator
#	- the dicom iod parser
#
# also checks the lengths of each dicom file
#
# also utilizes some utilities:
#
#	- dicom to raw convertor
#	- extended word pgm to raw convertor

converter="$1"
shift
topgm="$1"
shift
dumper="$1"
shift
dicomcopier="$1"
shift
dicomdumper="$1"
shift
dicomcreate="$1"
shift
dicomiod="$1"
shift
dc3toraw="$1"
shift
dc3topnm="$1"
shift
pgmtoraw="$1"
shift
lengthawk="$1"
shift
timecmd="$1"
shift


for testfile in $*
do
	if [ "`echo $testfile | egrep '.dump|.ioddesc'`" != "" ] ; then continue; fi

	echo "Testing file $testfile"
	echo "    Testing IOD parser on transfer syntax -m" \
	     " -o 1.2.840.10008.1.2.1 on $testfile"
	echo "        converting original to dicom"
	$timecmd "$converter -s 9999 -m -o 1.2.840.10008.1.2.1 \
		 $testfile /tmp/testconv.dc3"
	echo "        parsing iod of dicom output"
	$timecmd "$dicomiod -v -w /tmp/testconv.dc3 >$testfile.dc3.ioddesc 2>&1"

	for syntax in \
		"" "-m" \
		"-o 1.2.840.10008.1.2.1" "-m -o 1.2.840.10008.1.2.1" \
		"-o 1.2.840.10008.1.2.2" "-m -o 1.2.840.10008.1.2.2"
	do
		echo "    Testing transfer syntax $syntax on $testfile"
		echo "        converting original to dicom"
		$timecmd "$converter -s 9999 $syntax $testfile /tmp/testconv.dc3"
		echo "        checking length of original dicom"
		$timecmd "$dicomdumper /tmp/testconv.dc3 >/tmp/testconv.dump 2>&1"
		nawk -f $lengthawk </tmp/testconv.dump | grep -i "bad"
		echo "        copying dicom"
		$timecmd "$dicomcopier $syntax /tmp/testconv.dc3 /tmp/testconv2.dc3"
		echo "        checking length of copied dicom"
		$timecmd "$dicomdumper /tmp/testconv2.dc3 >/tmp/testconv2.dump 2>&1"
		nawk -f $lengthawk </tmp/testconv2.dump | grep -i "bad"
		echo "        comparing dicom"
		cmp /tmp/testconv.dc3 /tmp/testconv2.dc3
		echo "        comparing dicom dumps"
		diff /tmp/testconv.dump /tmp/testconv2.dump
		rm -f /tmp/testconv2.dump

		#echo "        creating copy from dicom dump"
		#$timecmd "$dicomcreate </tmp/testconv.dump >/tmp/testconv.cr.dc3"
		# don't binary compare due to presence of image data
		# cmp /tmp/testconv.dc3 /tmp/testconv.cr.dc3
		#echo "        comparing dump of created copy"
		#$timecmd "$dicomdumper /tmp/testconv.cr.dc3 2>/tmp/testconv.cr.dump"
		#diff -b /tmp/testconv.dump /tmp/testconv.cr.dump

		#rm -f /tmp/testconv.cr.dc3
		#rm -f /tmp/testconv.cr.dump

		rm -f /tmp/testconv.dump

		rm -f /tmp/testconv2.dc3
		echo "        converting dicom to raw"
		$timecmd "$dc3toraw /tmp/testconv.dc3 /tmp/testconv.raw"
		echo "        converting original to pgm"
		$timecmd "$topgm <$testfile >/tmp/testconv2.pgm"
		echo "        converting original pgm to raw"
		$timecmd "$pgmtoraw -l </tmp/testconv2.pgm >/tmp/testconv2.raw"
		echo "        comparing original raw and dicom raw"
		cmp /tmp/testconv.raw /tmp/testconv2.raw

		echo "        converting dicom to pgm"
		$timecmd "$dc3topnm /tmp/testconv.dc3 /tmp/testconv.dc3.pgm"
		#echo "        comparing original pgm and dicom pgm"
		#cmp /tmp/testconv2.pgm /tmp/testconv.dc3.pgm
		echo "        converting dicom pgm to raw"
		$timecmd "$pgmtoraw -l </tmp/testconv.dc3.pgm >/tmp/testconv.dc3.pgm.raw"
		echo "        comparing original pgm raw and dicom pgm raw"
		cmp /tmp/testconv2.raw /tmp/testconv.dc3.pgm.raw

		rm -f /tmp/testconv.dc3 /tmp/testconv.raw
		rm -f /tmp/testconv2.raw /tmp/testconv2.pgm
		rm -f /tmp/testconv.dc3.pgm /tmp/testconv.dc3.pgm.raw
	done

	if [ -f $testfile.dump ]
	then
		rm -f $testfile.dump.old
		mv $testfile.dump $testfile.dump.old
	fi
	echo "    Testing dumper on $testfile"
	$timecmd "$dumper $testfile >$testfile.dump 2>&1"
	if [ -f $testfile.dump.old ]
	then
		echo "    comparing old dump with new"
		diff $testfile.dump.old $testfile.dump
		rm -f $testfile.dump.old	
		# leave $testfile.dump around for next time
	else
		echo "    no old dump for comparison"
	fi
done


